@@ -265,10 +265,12 @@ EXPRESSION_METADATA = {
             exp.Array,
             exp.ArrayAgg,
             exp.ArrayAppend,
+            exp.ArrayCompact,
             exp.ArrayConcat,
             exp.ArrayConstructCompact,
             exp.ArrayPrepend,
             exp.ArrayRemove,
+            exp.ArraysZip,
             exp.ArrayUniqueAgg,
             exp.ArrayUnionAgg,
             exp.MapKeys,
@@ -288,6 +290,7 @@ EXPRESSION_METADATA = {
             exp.MD5NumberLower64,
             exp.MD5NumberUpper64,
             exp.Rand,
+            exp.Seq8,
             exp.Zipf,
         }
     },
@@ -357,7 +360,6 @@ EXPRESSION_METADATA = {
             exp.ApproximateSimilarity,
             exp.Asinh,
             exp.Atanh,
-            exp.Cbrt,
             exp.Cosh,
             exp.CosineDistance,
             exp.CovarPop,
@@ -385,6 +387,7 @@ EXPRESSION_METADATA = {
             exp.Asin,
             exp.Atan,
             exp.Atan2,
+            exp.Cbrt,
             exp.Cos,
             exp.Cot,
             exp.Degrees,
@@ -413,21 +416,17 @@ EXPRESSION_METADATA = {
     **{
         expr_type: {"returns": exp.DataType.Type.INT}
         for expr_type in {
-            exp.Ascii,
-            exp.BitLength,
             exp.ByteLength,
-            exp.Getbit,
             exp.Grouping,
             exp.Hour,
             exp.JarowinklerSimilarity,
-            exp.Length,
-            exp.Levenshtein,
             exp.MapSize,
             exp.Minute,
             exp.RtrimmedLength,
             exp.Second,
-            exp.StrPosition,
-            exp.Unicode,
+            exp.Seq1,
+            exp.Seq2,
+            exp.Seq4,
             exp.WidthBucket,
         }
     },
@@ -483,7 +482,6 @@ EXPRESSION_METADATA = {
             exp.Base64Encode,
             exp.CheckJson,
             exp.CheckXml,
-            exp.Chr,
             exp.Collate,
             exp.Collation,
             exp.CurrentAccount,
@@ -514,11 +512,8 @@ EXPRESSION_METADATA = {
             exp.RegexpReplace,
             exp.Repeat,
             exp.Replace,
-            exp.SHA,
-            exp.SHA2,
             exp.Soundex,
             exp.SoundexP123,
-            exp.Space,
             exp.SplitPart,
             exp.Translate,
             exp.TryBase64DecodeString,
