@@ -951,4 +951,6 @@ SELECT CURRENT_DATABASE()
 SELECT CURRENT_SCHEMAS(arg_bool)
 SELECT UNIFORM(1, 10, 5)
 SELECT UNIFORM(1, 10)
-SELECT CURRENT_TIMEZONE()
\ No newline at end of file
+SELECT CURRENT_TIMEZONE()
+SELECT NUMRANGE(1.1, 2.2) -|- NUMRANGE(2.2, 3.3)
+CREATE TABLE t (a VARCHAR, check INT)
